@@ -1,8 +1,9 @@
-use std::collections::HashMap;
-use std::convert::TryFrom;
+use indexmap::IndexMap;
 use std::rc::Rc;
 
-use crate::binaryxml::{XmlCdata, XmlElement, XmlStartElement, XmlStartNameSpace};
+use crate::binaryxml::{
+    BinaryXmlDocument, XmlCdata, XmlNodeType, XmlStartElement, XmlStartNameSpace,
+};
 use crate::stringpool::StringPool;
 use crate::ParseError;
 
@@ -13,30 +14,29 @@ pub struct XmlDocument {
 }
 
 impl XmlDocument {
-    pub(crate) fn new(
-        elements: Vec<XmlElement>,
-        string_pool: StringPool,
-        resource_map: Vec<u32>,
-    ) -> Result<Self, ParseError> {
-        let mut namespaces = HashMap::new();
+    pub(crate) fn new(binaryxml: BinaryXmlDocument) -> Result<Self, ParseError> {
+        let string_pool = binaryxml.string_pool;
+        let resource_map = binaryxml.resource_map;
+
+        let mut namespaces = IndexMap::new();
 
         let mut element_tracker: Vec<Element> = Vec::new();
-        for element in elements {
-            match element {
-                XmlElement::XmlStartNameSpace(e) => {
+        for node in binaryxml.elements {
+            match node.element {
+                XmlNodeType::XmlStartNameSpace(e) => {
                     let (uri, prefix) = Self::process_start_namespace(&e, &string_pool)?;
                     namespaces.insert(uri.clone(), prefix.clone());
                 }
-                XmlElement::XmlEndNameSpace(_) => {}
-                XmlElement::XmlStartElement(e) => {
+                XmlNodeType::XmlEndNameSpace(_) => {}
+                XmlNodeType::XmlStartElement(e) => {
                     element_tracker.push(Self::process_start_element(
                         &e,
                         &string_pool,
                         &namespaces,
-                        &resource_map,
+                        &resource_map.resource_ids,
                     )?);
                 }
-                XmlElement::XmlEndElement(_) => {
+                XmlNodeType::XmlEndElement(_) => {
                     let e = element_tracker.pop().unwrap();
 
                     if element_tracker.is_empty() {
@@ -50,7 +50,7 @@ impl XmlDocument {
                         .unwrap()
                         .insert_children(Node::Element(e));
                 }
-                XmlElement::XmlCdata(e) => {
+                XmlNodeType::XmlCdata(e) => {
                     let cdata = Self::process_cdata(&e, &string_pool)?;
                     element_tracker
                         .last_mut()
@@ -94,7 +94,7 @@ impl XmlDocument {
     fn process_start_element(
         e: &XmlStartElement,
         string_pool: &StringPool,
-        namespaces: &HashMap<Rc<String>, Rc<String>>,
+        namespaces: &IndexMap<Rc<String>, Rc<String>>,
         resource_map: &[u32],
     ) -> Result<Element, ParseError> {
         let ns = string_pool.get(usize::try_from(e.attr_ext.ns).unwrap());
@@ -105,13 +105,13 @@ impl XmlDocument {
             .ok_or(ParseError::StringNotFound(e.attr_ext.name))?;
         let name = (*name).clone();
 
-        let mut attributes: HashMap<String, String> = HashMap::new();
+        let mut attributes: IndexMap<String, String> = IndexMap::new();
         for attr in &e.attributes {
             let ns = string_pool.get(usize::try_from(attr.ns).unwrap());
             let name = string_pool
                 .get(usize::try_from(attr.name).unwrap())
                 .ok_or(ParseError::StringNotFound(attr.name))?;
-            let value = attr.typed_value.get_value(&string_pool);
+            let value = attr.typed_value.get_value(string_pool);
 
             let mut final_name = String::new();
             if !name.is_empty() {
@@ -153,14 +153,14 @@ pub enum Node {
 ///Struct representing an element within the parsed XML document.
 #[derive(Debug)]
 pub struct Element {
-    attributes: HashMap<String, String>,
+    attributes: IndexMap<String, String>,
     tag: String,
     children: Vec<Node>,
 }
 
 impl Element {
     ///Returns a map of attributes associated with the element.
-    pub fn get_attributes(&self) -> &HashMap<String, String> {
+    pub fn get_attributes(&self) -> &IndexMap<String, String> {
         &self.attributes
     }
 
